@@ -2,11 +2,24 @@
 #include "config.h"
 #endif
 
+#include "gtk-clutter-util.h"
+
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <clutter/clutter.h>
 
-#include "gtk-clutter-util.h"
+#if defined(HAVE_CLUTTER_GTK_X11)
+
+#include <clutter/x11/clutter-x11.h>
+#include <gdk/gdkx.h>
+
+#elif defined(HAVE_CLUTTER_GTK_WIN32)
+
+#include <clutter/clutter-win32.h>
+#include <gdk/gdkwin32.h>
+
+#endif /* HAVE_CLUTTER_GTK_{X11,WIN32} */
 
 /**
  * SECTION:gtk-clutter-util
@@ -321,26 +334,33 @@ gtk_clutter_texture_new_from_pixbuf (GdkPixbuf *pixbuf)
   return retval; 
 }
 
+GQuark
+gtk_clutter_texture_error_quark (void)
+{
+  return g_quark_from_static_string ("clutter-gtk-texture-error-quark");
+}
+
 /**
  * gtk_clutter_texture_set_from_pixbuf:
  * @texture: a #ClutterTexture
  * @pixbuf: a #GdkPixbuf
+ * @error: a return location for errors
  *
  * Sets the contents of @texture with a copy of @pixbuf.
  *
+ * Return value: %TRUE on success, %FALSE on failure.
+ *
  * Since: 0.8
  */
-void
+gboolean
 gtk_clutter_texture_set_from_pixbuf (ClutterTexture *texture,
-                                     GdkPixbuf      *pixbuf)
+                                     GdkPixbuf      *pixbuf,
+                                     GError        **error)
 {
-  GError *error;
+  g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), FALSE);
+  g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), FALSE);
 
-  g_return_if_fail (CLUTTER_IS_TEXTURE (texture));
-  g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
-
-  error = NULL;
-  clutter_texture_set_from_rgb_data (texture,
+  return clutter_texture_set_from_rgb_data (texture,
                                      gdk_pixbuf_get_pixels (pixbuf),
                                      gdk_pixbuf_get_has_alpha (pixbuf),
                                      gdk_pixbuf_get_width (pixbuf),
@@ -348,12 +368,7 @@ gtk_clutter_texture_set_from_pixbuf (ClutterTexture *texture,
                                      gdk_pixbuf_get_rowstride (pixbuf),
                                      gdk_pixbuf_get_has_alpha (pixbuf) ? 4 : 3,
                                      0,
-                                     &error);
-  if (error)
-    {
-      g_warning ("Unable to set the pixbuf: %s", error->message);
-      g_error_free (error);
-    }
+                                     error);
 }
 
 /**
@@ -399,33 +414,44 @@ gtk_clutter_texture_new_from_stock (GtkWidget   *widget,
  * @widget: a #GtkWidget
  * @stock_id: the stock id of the icon
  * @size: the size of the icon, or -1
+ * @error: a return location for errors
  *
  * Sets the contents of @texture using the stock icon @stock_id, as
  * rendered by @widget.
  *
+ * Return value: %TRUE on success, %FALSE on failure.
+ *
  * Since: 0.8
  */
-void
+gboolean
 gtk_clutter_texture_set_from_stock (ClutterTexture *texture,
                                     GtkWidget      *widget,
                                     const gchar    *stock_id,
-                                    GtkIconSize     size)
+                                    GtkIconSize     size,
+                                    GError        **error)
 {
   GdkPixbuf *pixbuf;
+  gboolean returnval;
 
-  g_return_if_fail (CLUTTER_IS_TEXTURE (texture));
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-  g_return_if_fail (stock_id != NULL);
-  g_return_if_fail (size > GTK_ICON_SIZE_INVALID || size == -1);
+  g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), FALSE);
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  g_return_val_if_fail (stock_id != NULL, FALSE);
+  g_return_val_if_fail ((size > GTK_ICON_SIZE_INVALID) || (size == -1), FALSE);
 
   pixbuf = gtk_widget_render_icon (widget, stock_id, size, NULL);
   if (!pixbuf)
-    pixbuf = gtk_widget_render_icon (widget,
-                                     GTK_STOCK_MISSING_IMAGE, size,
-                                     NULL);
+    {
+      g_set_error (error,
+                   CLUTTER_GTK_TEXTURE_ERROR,
+                   CLUTTER_GTK_TEXTURE_INVALID_STOCK_ID,
+                   "Stock ID '%s' not found", stock_id);
+      return FALSE;
+    }
 
-  gtk_clutter_texture_set_from_pixbuf (texture, pixbuf);
+  returnval = gtk_clutter_texture_set_from_pixbuf (texture, pixbuf, error);
   g_object_unref (pixbuf);
+
+  return returnval;
 }
 
 /**
@@ -511,28 +537,33 @@ gtk_clutter_texture_new_from_icon_name (GtkWidget   *widget,
  * @widget: a #GtkWidget or %NULL
  * @icon_name: the name of the icon
  * @size: the icon size or -1
+ * @error: a return location for errors
  *
  * Sets the contents of @texture using the @icon_name from the
  * current icon theme.
  *
+ * Return value: %TRUE on success, %FALSE on failure.
+ *
  * Since: 0.8
  */
-void
+gboolean
 gtk_clutter_texture_set_from_icon_name (ClutterTexture *texture,
                                         GtkWidget      *widget,
                                         const gchar    *icon_name,
-                                        GtkIconSize     size)
+                                        GtkIconSize     size,
+                                        GError        **error)
 {
+  GError *local_error = NULL;
   GtkSettings *settings;
   GtkIconTheme *icon_theme;
+  gboolean returnval;
   gint width, height;
   GdkPixbuf *pixbuf;
-  GError *error;
 
-  g_return_if_fail (CLUTTER_IS_TEXTURE (texture));
-  g_return_if_fail (widget == NULL || GTK_IS_WIDGET (widget));
-  g_return_if_fail (icon_name != NULL);
-  g_return_if_fail (size > GTK_ICON_SIZE_INVALID || size == -1);
+  g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), FALSE);
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  g_return_val_if_fail (icon_name != NULL, FALSE);
+  g_return_val_if_fail ((size > GTK_ICON_SIZE_INVALID) || (size == -1), FALSE);
 
   if (widget && gtk_widget_has_screen (widget))
     {
@@ -554,28 +585,119 @@ gtk_clutter_texture_set_from_icon_name (ClutterTexture *texture,
       width = height = 48;
     }
 
-  error = NULL;
   pixbuf = gtk_icon_theme_load_icon (icon_theme,
                                      icon_name,
                                      MIN (width, height), 0,
-                                     &error);
-  if (error)
+                                     &local_error);
+  if (local_error)
     {
-      g_warning ("Unable to load the icon `%s' from the theme: %s",
-                 icon_name,
-                 error->message);
-
-      g_error_free (error);
-
-      if (widget)
-        gtk_clutter_texture_set_from_stock (texture,
-                                      widget,
-                                      GTK_STOCK_MISSING_IMAGE,
-                                      size);
-      else
-        return;
+      g_propagate_error (error, local_error);
+      return FALSE;
     }
 
-  gtk_clutter_texture_set_from_pixbuf (texture, pixbuf);
+  returnval = gtk_clutter_texture_set_from_pixbuf (texture, pixbuf, error);
   g_object_unref (pixbuf);
+
+  return returnval;
+}
+
+/**
+ * gtk_clutter_init:
+ * @argc: pointer to the arguments count, or %NULL
+ * @argv: pointer to the arguments vector, or %NULL
+ *
+ * This function should be called instead of clutter_init() and
+ * gtk_init().
+ *
+ * Return value: %CLUTTER_INIT_SUCCESS on success, a negative integer
+ *   on failure.
+ *
+ * Since: 0.8
+ */
+ClutterInitError
+gtk_clutter_init (int    *argc,
+                  char ***argv)
+{
+  if (!gtk_init_check (argc, argv))
+    return CLUTTER_INIT_ERROR_GTK;
+
+#if defined(HAVE_CLUTTER_GTK_X11)
+  clutter_x11_set_display (gdk_x11_display_get_xdisplay (gdk_display_get_default ()));
+  clutter_x11_disable_event_retrieval ();
+#elif defined(HAVE_CLUTTER_GTK_WIN32)
+  clutter_win32_disable_event_retrieval ();
+#endif /* HAVE_CLUTTER_GTK_{X11,WIN32} */
+
+  return clutter_init (argc, argv);
+}
+
+/**
+ * gtk_clutter_init_with_args:
+ * @argc: a pointer to the number of command line arguments.
+ * @argv: a pointer to the array of command line arguments.
+ * @parameter_string: a string which is displayed in
+ *    the first line of <option>--help</option> output, after
+ *    <literal><replaceable>programname</replaceable> [OPTION...]</literal>
+ * @entries: a %NULL-terminated array of #GOptionEntry<!-- -->s
+ *    describing the options of your program
+ * @translation_domain: a translation domain to use for translating
+ *    the <option>--help</option> output for the options in @entries
+ *    with gettext(), or %NULL
+ * @error: a return location for errors
+ *
+ * This function should be called instead of clutter_init() and
+ * gtk_init_with_args().
+ *
+ * Return value: %CLUTTER_INIT_SUCCESS on success, a negative integer
+ *   on failure.
+ *
+ * Since: 0.10
+ */
+ClutterInitError
+gtk_clutter_init_with_args (int            *argc,
+                            char         ***argv,
+                            const char     *parameter_string,
+                            GOptionEntry   *entries,
+                            const char     *translation_domain,
+                            GError        **error)
+{
+  GOptionGroup *gtk_group, *clutter_group;
+  GOptionContext *context;
+  gboolean res;
+
+  /* we let gtk+ open the display */
+  gtk_group = gtk_get_option_group (TRUE);
+
+  /* and we prevent clutter from doing so too */
+  clutter_group = clutter_get_option_group_without_init ();
+
+  context = g_option_context_new (parameter_string);
+
+  g_option_context_add_group (context, gtk_group);
+  g_option_context_add_group (context, clutter_group);
+
+  if (entries)
+    g_option_context_add_main_entries (context, entries, translation_domain);
+
+  res = g_option_context_parse (context, argc, argv, error);
+  g_option_context_free (context);
+
+  if (!res)
+    return CLUTTER_INIT_ERROR_GTK;
+
+#if defined(GDK_WINDOWING_X11)
+  clutter_x11_set_display (gdk_x11_display_get_xdisplay (gdk_display_get_default ()));
+  clutter_x11_disable_event_retrieval ();
+#elif defined(GDK_WINDOWING_WIN32)
+  clutter_win32_disable_event_retrieval ();
+#endif /* GDK_WINDOWING_{X11,WIN32} */
+
+  /* this is required since parsing clutter's option group did not
+   * complete the initialization process
+   */
+  return clutter_init_with_args (argc, argv,
+                                 NULL,
+                                 NULL,
+                                 NULL,
+                                 error);
 }
